/*
* Copyright (c) 2003, University of Maryland 
* Copyright (c) 2003, Center for Satellite and Hybrid Communication Networks 
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the University of Maryland/Center for Satellite
*     and Hybrid Communication Networks nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

/* Authors: 
*     Jonathan Polley (jpolley@isr.umd.edu),
*     Dionysus Blazakis (dblaze@isr.umd.edu),
*     Jonathan McGee (mcgee@isr.umd.edu),
*     Daniel Rusk (ruskd@isr.umd.edu),
*     Karl Seamon (kks@isr.umd.edu),
*     Manish Karir (karir@isr.umd.edu)
*/

#ifndef ATEMU_DEFS_H
#define ATEMU_DEFS_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <inttypes.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#ifndef ERROR_T
#define ERROR_T
typedef int error_t;
#endif

#include "os.h"

/*************************************************/
/* Helper Macros */
/*************************************************/

/* these are for decoding words stored in the emulator.  Needs endian. */
#ifdef ATEMU_LITTLE_ENDIAN

#define TOP_BYTE(x) (((x) >> 8) & 0xff)
#define BOTTOM_BYTE(x) ((x) & 0xff)

#define CONVERT_ADDR(x) (x)

#define CONVERT_2BYTE(x) (x)
#define CONVERT_4BYTE(x) (x)
#define CONVERT_8BYTE(x) (x)

#define BYTE_N(x,n) (((x) >> ((sizeof(x) - (n) - 1) * 8)) & 0xff)

#else  /* big endian here */

#define TOP_BYTE(x) ((x) & 0xff)
#define BOTTOM_BYTE(x) (((x) >> 8) & 0xff)

#define CONVERT_ADDR(x) ((TOP_BYTE(x) << 8) | (BOTTOM_BYTE(x)))

#define BYTE_N(x,n) (((x) >> ((n) * 8)) & 0xff)

#define CONVERT_2BYTE(x) CONVERT_ADDR(x)

#define CONVERT_4BYTE(x) ((BYTE_N((x), 3)) | (BYTE_N((x), 2) << 8) \
			| (BYTE_N((x), 1) << 16) | (BYTE_N((x), 0) << 24))

#define CONVERT_8BYTE(x) ((BYTE_N((x), 7)) | (BYTE_N((x), 6) << 8) \
			| (BYTE_N((x), 5) << 16) | (BYTE_N((x), 4) << 24) \
			| (BYTE_N((x), 3) << 32) | (BYTE_N((x), 2) << 40) \
			| (BYTE_N((x), 1) << 48) | (BYTE_N((x), 0) << 56))

#endif

/* these are for packing and unpacking host byte order numbers
 * for use in mathematics.  Not endian sensitive. */
#define HIGH_BYTE_16(x) (((x) >> 8) & 0xff)
#define LOW_BYTE_16(x) ((x) & 0xff)
#define PACK_INT_16(h,l) ((((uint16_t)h) << 8) | l)

#define HIGH_BYTE_24(x) (((x) >> 16) & 0xff)
#define MID_BYTE_24(x) (((x) >> 8) & 0xff)
#define LOW_BYTE_24(x) ((x) & 0xff)
#define PACK_INT_24(h,m,l) ((((uint32_t)h) << 16) | (((uint32_t)m) << 8) | l)

/* NIBBLE shifting is within a byte, so not endian specific */
#define LOW_NIBBLE(x) ((x) & 0xf)
#define HIGH_NIBBLE(x) (((x) >> 4 ) & 0xf)

#define SET_FLAG(byte,flag) ((byte) |= (flag))
#define CLR_FLAG(byte,flag) ((byte) &= (~(flag)))
#define GET_FLAG(byte,flag) (((byte) & (flag)) ? 1 : 0)

#endif /* ATEMU_DEFS_H */
