/*
* Copyright (c) 2003, University of Maryland 
* Copyright (c) 2003, Center for Satellite and Hybrid Communication Networks 
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the University of Maryland/Center for Satellite
*     and Hybrid Communication Networks nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

/* Authors: 
*     Jonathan Polley (jpolley@isr.umd.edu),
*     Dionysus Blazakis (dblaze@isr.umd.edu),
*     Jonathan McGee (mcgee@isr.umd.edu),
*     Daniel Rusk (ruskd@isr.umd.edu),
*     Karl Seamon (kks@isr.umd.edu),
*     Manish Karir (karir@isr.umd.edu)
*/

#ifndef ATEMU_USART_H
#define ATEMU_USART_H

#include "../device.h"

/* All the control register bits */
#define USART_CONTROLA_RXC0   (uint8_t)0x80
#define USART_CONTROLA_TXC0   (uint8_t)0x40
#define USART_CONTROLA_UDRE0  (uint8_t)0x20
#define USART_CONTROLA_FE0    (uint8_t)0x10
#define USART_CONTROLA_DOR0   (uint8_t)0x08
#define USART_CONTROLA_UPE0   (uint8_t)0x04
#define USART_CONTROLA_U2X0   (uint8_t)0x02
#define USART_CONTROLA_MPCM0  (uint8_t)0x01
    /* Bitmask of those bits that are status and not control bits */
#define USART_CONTROLA_STATUS (uint8_t)0xFC

#define USART_CONTROLB_RXCIE0 (uint8_t)0x80
#define USART_CONTROLB_TXCIE0 (uint8_t)0x40
#define USART_CONTROLB_UDRIE0 (uint8_t)0x20
#define USART_CONTROLB_RXEN0  (uint8_t)0x10
#define USART_CONTROLB_TXEN0  (uint8_t)0x08
#define USART_CONTROLB_UCXZ02 (uint8_t)0x04
#define USART_CONTROLB_RXB80  (uint8_t)0x02
#define USART_CONTROLB_TXB80  (uint8_t)0x01
    /* Bitmask of those bits that are status and not control bits */
#define USART_CONTROLB_STATUS (uint8_t)0x02

#define USART_CONTROLC_UMSEL0 (uint8_t)0x40
#define USART_CONTROLC_UMP01  (uint8_t)0x20
#define USART_CONTROLC_UMP00  (uint8_t)0x10
#define USART_CONTROLC_UMP0   (USART_CONTROLC_UMP01|USART_CONTROLC_UMP00)
#define USART_CONTROLC_UMP0_DISABLE 0
#define USART_CONTROLC_UMP0_RESERVE USART_CONTROLC_UMP00
#define USART_CONTROLC_UMP0_EVEN    USART_CONTROLC_UMP01
#define USART_CONTROLC_UMP0_ODD     USART_CONTROLC_UMP0
#define USART_CONTROLC_USBS0  (uint8_t)0x08
#define USART_CONTROLC_UCSZ01 (uint8_t)0x04
#define USART_CONTROLC_UCSZ00 (uint8_t)0x02
#define USART_CONTROLC_UCSZ0  (USART_CONTROLC_UCSZ01|USART_CONTROLC_UCSZ00)
#define USART_CONTROLC_UCSZ0_5BIT 0
#define USART_CONTROLC_UCSZ0_6BIT USART_CONTROLC_UCSZ00
#define USART_CONTROLC_UCSZ0_7BIT USART_CONTROLC_UCSZ01
#define USART_CONTROLC_UCSZ0_8BIT USART_CONTROLC_UCSZ0
#define USART_CONTROLC_UCPOL0 (uint8_t)0x01
    /* Bitmask of those bits that are status and not control bits */
#define USART_CONTROLC_STATUS (uint8_t)0x80

typedef struct {
    device_obj    device;

    register_obj  reg_data;
    register_obj  reg_controlA;
    register_obj  reg_controlB;
    register_obj  reg_controlC;
    register_obj  reg_baudhigh;
    register_obj  reg_baudlow;

    clock_obj     clk_tick;
    interrupt_obj int_rx;
    interrupt_obj int_udre;
    interrupt_obj int_tx;

    /* Parameteres */
        /* System clock frequency (Hertz) */
    uint32_t fosc;
        /* Filename to write to */
    char *filename;
    int file;
        /* File mode */
    int filemode;
    int canwrite;
    int canread;

    /* Values of the various registers */
    uint8_t controlA;
    uint8_t controlB;
    uint8_t controlC;
    uint16_t baud;

    /* Data buffers (with error status) */
    /* The current emulator architecture is not well designed for this
     * uint16_t recvtop;
     * uint16_t recvbottom;
     */
        /* Data byte buffered for transmission */
    uint16_t dataout;
        /* Delay until dataout can be written to file and next byte read */
    uint16_t datadelay;
        /* The "true" baud */
    uint16_t databaud;
} usart_device;

#endif /* ATEMU_USART_H */
