/*
* Copyright (c) 2003, University of Maryland 
* Copyright (c) 2003, Center for Satellite and Hybrid Communication Networks 
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the University of Maryland/Center for Satellite
*     and Hybrid Communication Networks nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

/* Authors: 
*     Jonathan Polley (jpolley@isr.umd.edu),
*     Dionysus Blazakis (dblaze@isr.umd.edu),
*     Jonathan McGee (mcgee@isr.umd.edu),
*     Daniel Rusk (ruskd@isr.umd.edu),
*     Karl Seamon (kks@isr.umd.edu),
*     Manish Karir (karir@isr.umd.edu)
*/

#include  <fcntl.h>
#include  <stdio.h>
#include  <unistd.h>
#include  <libelf.h>
#include  <stdlib.h>
#include  <string.h>

#include "defs.h"

error_t elf_load( uint8_t *flash, int size, uint32_t *eaddr, char *filename )
{
    Elf *          elf;
    Elf32_Ehdr *   ehdr;
    Elf32_Phdr *   phdr;

    int            fd;
    unsigned int   cnt;

    /* Open the input file */
    if ((fd = open(filename, O_RDONLY)) == -1)
        return -1;

    /* Obtain the ELF descriptor */
    (void) elf_version(EV_CURRENT);
    if ((elf = elf_begin(fd, ELF_C_READ, NULL)) == NULL){
        elf_end( elf );
        close(fd);
        return -1;
    }

    if ((phdr = elf32_getphdr(elf)) == NULL){
        elf_end( elf );
        close(fd);
        return -1;
    }

    /* Obtain the .shstrtab data buffer */
    if ( (ehdr = elf32_getehdr(elf)) == NULL){
        elf_end( elf );
        close(fd);
        return -1;
    }

//    *eaddr = (uint32_t) ehdr->e_entry;
    *eaddr = 0x00;

    for(cnt = 0; cnt < ehdr->e_phnum; cnt++ )
    {
        if ( phdr[cnt].p_type == PT_LOAD ) {
            if ( phdr[cnt].p_filesz != 0 )
            {
                if ( lseek( fd, phdr[cnt].p_offset, SEEK_SET ) != phdr[cnt].p_offset )
                {
                    printf( "Coudln't seek! [%d]\n", cnt );
                    elf_end( elf );
                    close(fd);
                    return -1;
                }

                if ( (phdr[cnt].p_paddr + phdr[cnt].p_filesz) > size )
                {
                    printf( "Too big! [%d]\n", cnt );
                    elf_end( elf );
                    close(fd);
                    return -1;
                }

                if ( read( fd, flash + phdr[cnt].p_paddr, phdr[cnt].p_filesz ) != phdr[cnt].p_filesz )
                {
                    printf( "Error reading! [%d]\n", cnt );
                    elf_end( elf );
                    close(fd);
                    return -1;
                }
            }
        }
    }

    elf_end( elf );
    close(fd);

    return 0;
}
