/*
* Copyright (c) 2003, University of Maryland 
* Copyright (c) 2003, Center for Satellite and Hybrid Communication Networks 
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the University of Maryland/Center for Satellite
*     and Hybrid Communication Networks nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

/* Authors: 
*     Jonathan Polley (jpolley@isr.umd.edu),
*     Dionysus Blazakis (dblaze@isr.umd.edu),
*     Jonathan McGee (mcgee@isr.umd.edu),
*     Daniel Rusk (ruskd@isr.umd.edu),
*     Karl Seamon (kks@isr.umd.edu),
*     Manish Karir (karir@isr.umd.edu)
*/

#ifndef FUNC_DATA_H
#define FUNC_DATA_H

#include <inttypes.h>
#include <glib.h>

#include "type_data.h"

enum {
    LOCAL_TYPE_REG,
    LOCAL_TYPE_STACK
};

struct local {
    char * name;

    int is_param;

    int type;

    /* Depends on type: if stack, gives stack offset
     *                  if reg, gives reg number
     */
    int location;

    struct type_data *td;

    uint8_t *value;
    int type_style;

    int id;
};

struct c_line {
    long int addr;
    long int line;

};

struct func_data {
    GSList *locals;
    GSList *c_lines;

    char * name;

    /* begin is inclusive, end is exclusive... could be an 
       empty interval if stabs are wrong
     */
    long addr_begin;
    long addr_end;
};

struct func_data *create_func_data( char * name, int beg, int end );
void add_local_to_func( struct func_data * fd, struct local * loc );
void add_c_line_to_func( struct func_data * fd, struct c_line * cl );
int func_data_compare( const void * a, const void * b );
void free_func_data( void * a );

struct local * create_local( char * name, int is_param, int type, 
    int location, struct type_data *td, int id );
void free_local( struct local * loc );

struct c_line * create_c_line( long int addr, long int line );
void free_c_line( struct c_line * cl );

#endif
