/*
* Copyright (c) 2003, University of Maryland 
* Copyright (c) 2003, Center for Satellite and Hybrid Communication Networks 
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the University of Maryland/Center for Satellite
*     and Hybrid Communication Networks nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

/* Authors: 
*     Jonathan Polley (jpolley@isr.umd.edu),
*     Dionysus Blazakis (dblaze@isr.umd.edu),
*     Jonathan McGee (mcgee@isr.umd.edu),
*     Daniel Rusk (ruskd@isr.umd.edu),
*     Karl Seamon (kks@isr.umd.edu),
*     Manish Karir (karir@isr.umd.edu)
*/

#ifndef ATEMU_TABLE_H
#define ATEMU_TABLE_H

/* table.h
 *
 * defines routines for a (relatively) fast access general purpose "table"
 * create, destroy, store, delete, lookup.  The usual.
 *
 * Comparisons are done with strcmp-style function pointers.
 *
 * at this point. I think I'll implement it as a bst.
 */

#ifndef ERROR_T
#define ERROR_T
typedef int error_t;
#endif

typedef int (table_cmp_func)(const void *a, const void *b);
typedef void (table_free_func)(void *a);
typedef void (table_apply_func)(void *a, void *extra);

typedef struct table_t table_t;

table_t *table_create(table_cmp_func *cmpfunc);

/* deletes the tree */
/* if freefunc != NULL, calls it on the data (presumably to free it) */
error_t table_destroy(table_t *table, table_free_func *freefunc);

/* it does not make copies of the data.  Beware! */
error_t table_store(table_t *table, void *data);

/* deletes the first thing it finds that the compare function says is equal */
/* if freefunc != NULL, calls it on the data (presumably to free it) */
error_t table_delete(table_t *table, void *data, table_free_func *freefunc);

/* returns NULL if it didn't find it, data if it did */
void *table_find(table_t *table, void *data);

/* at each node, the data at that node is passed to applyfunc along */
/* with the extra. */
void table_foreach(table_t *t, table_apply_func *applyfunc, void *extra);

#endif

